 #
 #   $Id: Config.pm,v 1.11 2007/02/06 13:59:07 zarahg Exp $
 #
 #   This  file  is  part of the  PodWiki  web-based authoring tool.
 #
 #   By  accessing  this software,  PodWiki,  you are  duly informed
 #   of and  agree to be  bound  by the  conditions  described below
 #   in this notice:
 #
 #   This software product, PodWiki,  is developed by  Thomas Linden
 #   and   copyrighted  (C) 2003-2007   by  Thomas Linden,  with all
 #   rights reserved.
 #
 #   There is  no charge for  PodWiki software. You can redistribute
 #   it and/or modify it under  the terms of the  GNU General Public
 #   License, which is incorporated by reference herein.
 #
 #   PodWiki is distributed WITHOUT ANY WARRANTY,IMPLIED OR EXPRESS,
 #   OF  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that
 #   the  use of it will not infringe on any third party's intellec-
 #   tual property rights.
 #
 #   You  should  have  received a  copy  of  the GNU General Public
 #   License along with PodWiki.  Copies  can also be obtained from:
 #
 #     http://www.gnu.org/copyleft/gpl.html
 #
 #   or by writing to:
 #
 #     Free Software Foundation, Inc.
 #     59 Temple Place, Suite 330
 #     Boston, MA 02111-1307
 #     USA
 #
 #   Or contact:
 #
 #    "Thomas Linden" <tom@daemon.de>
 #
 #   Additional Copyrights:
 #
 #

package PodWiki::Config;

use lib qw(. lib);

$PodWiki::Config::VERSION = "1.4";

use strict;
use Carp;
use Exporter;

use vars qw(@ISA @EXPORT @EXPORT_OK @EXPORT_TAGS $AUTOLOAD);

@ISA=qw(Exporter);
@EXPORT=qw();
@EXPORT_OK=qw();
@EXPORT_TAGS=();


sub new {
  my($this, %param) = @_;
  my $class = ref($this) || $this;
  my $self = \%param;
  bless($self, $class);
  return $self;
}

sub intermap {
  my($this, $map) = @_;
  local $_;
 
  my %map;
  if(-s $map) {
    open MAP, "<$map" or die "Could not open intermap file: $map: $!\n";
    while (<MAP>) {
      chomp;
      my($wiki, $uri) = split /\s\s*/;
      $map{$wiki} = $uri;
    }
    close MAP;
    return %map;
  }
}

sub getcomments {
  my($this) = @_;
  return $this->{_comment};
}

sub fetch {
  my($this) = @_;
  local $_;

  my %bool = (yes => 1, no => 0, on => 1, off => 0);

  open F, "<$this->{File}" or croak "Could not open Config file: $this->{File}: $!\n";
  my $comment = "";
  while (<F>) {
    chomp;
    next if /^\s*#\s*$/; # skip empty comments
    next if /^\s*#\s*.*\-sh\-/; # skip emacs hook
    if(/^\s*#\s*(.*)$/) {
      $comment .= "\n" . $1;
      next;
    }
    next if /^\s*$/; # skip empty lines
    s/^\s*//;        # remove redundant whitespaces
    s/\s*$//;        # ditto
    my ($opt, $value) = split /\s*=\s*/, $_, 2;
    $this->{$opt} = $value || "";  # pre-initialize vars, if empty. (to make "use strict" happy!)
    if (exists $bool{$this->{$opt}}) {
      $this->{$opt} = $bool{$this->{$opt}};
    }
    if($comment) {
      $this->{_comment}->{$opt} = $comment;
      $comment = "";
    }
    else {
      $this->{_comment}->{$opt} = "";
    }
  }
  close F;

  my @required = qw(data imgdir gfx rcsbindir templates);
  foreach my $opt (@required) {
    if(! exists $this->{$opt}) {
      croak "required config option not found: \"$opt\"!";
    }
  }
}


1;
