 #
 #   $Id: BBcode.pm,v 1.2 2007/02/12 16:30:14 zarahg Exp $
 #
 #   This  file  is  part of the  PodWiki  web-based authoring tool.
 #
 #   By  accessing  this software,  PodWiki,  you are  duly informed
 #   of and  agree to be  bound  by the  conditions  described below
 #   in this notice:
 #
 #   This software product, PodWiki,  is developed by  Thomas Linden
 #   and   copyrighted  (C) 2003-2007   by  Thomas Linden,  with all
 #   rights reserved.
 #
 #   There is  no charge for  PodWiki software. You can redistribute
 #   it and/or modify it under  the terms of the  GNU General Public
 #   License, which is incorporated by reference herein.
 #
 #   PodWiki is distributed WITHOUT ANY WARRANTY,IMPLIED OR EXPRESS,
 #   OF  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that
 #   the  use of it will not infringe on any third party's intellec-
 #   tual property rights.
 #
 #   You  should  have  received a  copy  of  the GNU General Public
 #   License along with PodWiki.  Copies  can also be obtained from:
 #
 #     http://www.gnu.org/copyleft/gpl.html
 #
 #   or by writing to:
 #
 #     Free Software Foundation, Inc.
 #     59 Temple Place, Suite 330
 #     Boston, MA 02111-1307
 #     USA
 #
 #   Or contact:
 #
 #    "Thomas Linden" <tom@daemon.de>
 #
 #   Additional Copyrights:
 #
 #    Copyright 2004 by David Bright
 #


package PodWiki::Formatter::BBcode;

$PodWiki::Formatter::BBcode::VERSION = "1.00";

use vars qw(@ISA @EXPORT @EXPORT_OK @EXPORT_TAGS @docpages);
use vars qw($me $config $page);

@ISA=qw(Exporter PodWiki::Runner PodWiki::Formatter);
@EXPORT=qw();
@EXPORT_OK=qw();
@EXPORT_TAGS=();

sub new {
  my($this, %param) = @_;
  my $class = ref($this) || $this;
  my $self = \%param;
  $config = $self->{config};
  bless($self, $class);
  return $self;
}

sub tags {
  my($this) = @_;
  return qw(bbcode);
}

sub format {
  my($this, $text) = @_;
    my @nodes = split /\n{2,}/, $text;
  foreach my $node (@nodes) {
    next if($node =~ /^\s*$/s);
    print "<p>" . $this->bbcode($node) . "</p>";
  } 
}

sub bbcode {
  my($this, $text) = @_;

  local $_ = $text;


  my $quotecss = "border-left: 3px solid #c4c4c4; padding-left: 20px;";
  my $wrotecss = "border-bottom: 1px solid #c4c4c4; margin-bottom: 20px; font-style: italic;";

  s/\[url=(.+?)\](.+?)\[\/url\]/$this->pod_link("$2|$1")/egsi;
  s/\[url\](.+?)\[\/url\]/$this->pod_link($1)/gsi;
  s/\[img[^\]]*\](.+?)\[\/img\]/<img src="$1"\/>/gsi;

  s/\[\/quote\]/<\/blockquote>/gsi;
  s/\[quote.*author=(.+?) .*\]/<blockquote style="$quotecss"><div style="$wrotecss">$1 schrieb:<\/div>/gsi;
  s/\[quote\]/<blockquote style="$quotecss">/gsi;

  my %map = (b => "strong", i => "i", u => "u", pre => "pre", 
             tt => "tt", code => "code", sub => "sub", sup => "sup",
             s => "strike");

  my $reg = join "|", keys %map;

  s/\[($reg)\]/<$map{$1}>/gsi;
  s/\[\/($reg)\]/<\/$map{$1}>/gsi;

  %map = (color => "color: %s;", size => "font-size: %s", font => "font-face: %s");
  $reg = join "|", keys %map;

  s/\[($reg)=(.+?)\]/sprintf (qq(<span style="$map{$1}">), $2)/gesi;
  s/\[\/($reg)\]/<\/$map{$1}>/gsi;

  s/\[center\]/<div align="center">/gsi;
  s/\[\/center\]/<\/div>/gsi;

  s/\[table\]/<table border="1" cellspacing="3" cellpadding="3" width="100%">/gsi;
  s/\[\/table\]/<\/table>/gsi;
  s/\[(tr|td)\]/<$1>/gsi;
  s/\[\/(tr|td)\]/<\/$1>/gsi;

  my $linkProtocolPattern = "(http|ftp|gopher|news|file|https|telnet)";
  s!(^|[\-\*\s\(])($linkProtocolPattern\:([^\s\<\>\"]+[^\s\.\,\!\?\;\:\)\<]))!$1<a href="http://anonym.to/?$2">$2</a>!gsi;

  return $_;
}

sub html2pre {
  my ($this, $text) = @_;
  local $_ = $text;
  s/&/&amp;/g;
  s/</&lt;/g;
  s/>/&gt;/g;
  return $_;
}


1;
