 #
 #   $Id: Creole.pm,v 1.5 2007/01/29 13:47:32 zarahg Exp $
 #
 #   This  file  is  part of the  PodWiki  web-based authoring tool.
 #
 #   By  accessing  this software,  PodWiki,  you are  duly informed
 #   of and  agree to be  bound  by the  conditions  described below
 #   in this notice:
 #
 #   This software product, PodWiki,  is developed by  Thomas Linden
 #   and   copyrighted  (C) 2003-2007   by  Thomas Linden,  with all
 #   rights reserved.
 #
 #   There is  no charge for  PodWiki software. You can redistribute
 #   it and/or modify it under  the terms of the  GNU General Public
 #   License, which is incorporated by reference herein.
 #
 #   PodWiki is distributed WITHOUT ANY WARRANTY,IMPLIED OR EXPRESS,
 #   OF  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that
 #   the  use of it will not infringe on any third party's intellec-
 #   tual property rights.
 #
 #   You  should  have  received a  copy  of  the GNU General Public
 #   License along with PodWiki.  Copies  can also be obtained from:
 #
 #     http://www.gnu.org/copyleft/gpl.html
 #
 #   or by writing to:
 #
 #     Free Software Foundation, Inc.
 #     59 Temple Place, Suite 330
 #     Boston, MA 02111-1307
 #     USA
 #
 #   Or contact:
 #
 #    "Thomas Linden" <tom@daemon.de>
 #
 #   Additional Copyrights:
 #
 #    Copyright 2004 by David Bright
 #


package PodWiki::Formatter::Creole;

$PodWiki::Formatter::Creole::VERSION = "1.00";

use vars qw(@ISA @EXPORT @EXPORT_OK @EXPORT_TAGS @docpages);
use vars qw($me $config $page);

@ISA=qw(Exporter PodWiki::Runner PodWiki::Formatter);
@EXPORT=qw();
@EXPORT_OK=qw();
@EXPORT_TAGS=();

sub new {
  my($this, %param) = @_;
  my $class = ref($this) || $this;
  my $self = \%param;
  $config = $self->{config};
  bless($self, $class);
  return $self;
}

sub tags {
  my($this) = @_;
  return qw(creole);
}

sub format {
  my($this, $text) = @_;

  # 1st, look 4 pre stuff
  my $now = time;
  my %pre = ();
  $text =~ s/\{\{\{(.+?)\}\}\}/
              $now++;
              $pre{$now} = $this->html2pre($1);
              $now;
           /gex;

  # now split into paragraphs and render each
  my $output;
  my @nodes = split /\n{2,}/, $text;
  foreach my $node (@nodes) {
    next if($node =~ /^\s*$/s);
    if($node =~ /^\s*[\*\#]+?\s/m) {
      foreach my $node($this->split_node($node)) {
        next if($node =~ /^\s*$/s);
        $output .= "<p>" . $this->render_creole($node) . "</p>";
      }
    }
    else {
      $output .= "<p>" . $this->render_creole($node) . "</p>";
    }
  }

  # re-insert the pre stuff, if any
  foreach my $tag (keys %pre) {
     $output =~ s/$tag/$pre{$tag}/;
  }

  print $output;
}


sub split_node {
  # produce sub-nodes from lists and tables
  my ($this, $node) = @_;
  my (@para, $list, $text, $lasttype);
  if($node =~ /^\s*([\*\#\|])/) {
    $lasttype = $1;
  }
  foreach (split /\n/, $node) {
    $_ .= "\n";
    if (/^\s*([\*\#])([\*\#]*?)\s/) {
      my $type = $1;
      if($list) {
        if ($type ne $lasttype) {
          push @para, $list;
          $list = $_;
        }
        else {
          $list .= $_;
        }
      }
      elsif($text) {
        push @para, $text;
        $text = "";
        $list = $_;
      }
      else {
        $list = $_;
      }
      $lasttype = $type;
    }
    else {
      if($list) {
        push @para, $list;
        $list = "";
        $text = $_;
      }
      elsif($text) {
        $text .= $_;
      }
      else {
        $text = $_;
      }
    }
  }
  if ($list) { push @para, $list; }
  if ($text) { push @para, $text; }
  return @para;
}

sub getdepth {
  my ($this, $length, $text) = @_;
  my $depth = length($text);
  $depth = 6 if $depth > 6;
  $depth = 2 if $depth < 2;
  return "<h$depth>$text<\/h$depth>";
}

sub render_list {
     my($this, $type, @list) = @_;
     my $lastlevel = 1;
     my $list = "";
     $list .= "<$type>\n";
     foreach my $item (@list) {
       if($item->{level} > $lastlevel) {
         for (my $num = $lastlevel; $num < $item->{level}; $num++) {
            # print as many openers as required
            $list .= "<$type>\n";
         }
       }                                 
       elsif($item->{level} < $lastlevel) {
         for (my $num = $item->{level}; $num < $lastlevel; $num++) {
            # print as many closers as required
            $list .= "</$type>\n";
         }
       }                                     
       $list .= "<li>" . $item->{content} . "</li>\n";
       $lastlevel = $item->{level};
     }
     #$list .= "</$type>\n";
     for (my $num = $item->{level}; $num < $lastlevel; $num++) {
       # print as many closers as required
       $list .= "</$type>\n";
     }
     return $list;
}

sub render_creole {
  my($this, $node) = @_;
  local $_ = $node;

  # print "<pre>$_</pre>";

  my @bullet_list;
  s/^\s*(\*+?)\s\s*(.*)$/
     my $level = length($1);
     push @bullet_list, { level => $level, content => $2 };
  /mgex;

  if(@bullet_list) {
    $_ = $this->render_list("ul", @bullet_list);
  }
  else {
    my @number_list;
    s/^\s*(\#+?)\s\s*(.*)$/
      my $level = length($1);
      push @number_list, { level => $level, content => $2 };
    /mgex;
    if(@number_list) {
      $_ = $this->render_list("ol", @number_list);
    }
  }

  if(/^\s*\|/) {
    # table node
    my ($head, $body) = split /\n/, $_, 2;
    my @rows = split /\n/, $body;
    $head =~ s/^\s*\|//;
    $head =~ s/\|?$/ /;
    my @head_cells = split /\|/, $head;
    my @row_rows;
    foreach my $row (@rows) {
      $row =~ s/^\s*\|//;
      $row =~ s/\|?$/ /;
      my @cells = split /\|/, $row;
      push @row_rows, \@cells;
    }
    $_ = $this->gettable(\@head_cells, \@row_rows);
  }

  # heading
  s/\G(\s*\n)*(==+)[ \t]*(.*?)[ \t]*=*[ \t]*(\n|\Z)/$this->getdepth($2,$3)/ge;

  # horizontal line
  s/\G(\s*\n)*[ \t]*----+[ \t]*\n?/<hr noshade\/>/g;
  s/\G\s*\n----+[ \t]*\n?/<hr noshade\/>/g; 

  # clean-up by rule
  # **// bold italic **// is not allowed. the first occuring tag wins.
  s|\*\*(.*?)//(.*?)\*\*(.*?)//|**$1$2**//$3//|g;
  s|//(.*?)\*\*(.*?)//(.*?)\*\*|//$1$2//**$3**|g;

  # bold
  s/\*\*([^\*\*]*)\*\*/<strong>$1<\/strong>/g;

  my @proto = qw(http ftp gopher news file https telnet);
  my $linkProtocolPattern = "(" . join ("|", @proto) . ")";
  my $behind4italic = join "", map { "(?<!" . $_ . ":)" } @proto;

  # italic
  s#$behind4italic//([^//]*)//#<i>$1</i>#g;

  # embedded url
  s#(?<!\[\[)($linkProtocolPattern\:([^\s\<\>\"]+[^\s\.\,\!\?\;\:\)\<]))(?!\]\])#$this->pod_link($1)#ge;
  
  # pages
  my $pagePattern = '[\.: A-Za-z0-9_\-\x80-\xFF]*[A-Za-z0-9_\x80-\xFF]';
  s!\[\[($pagePattern)\]\]!$this->pod_link($1)!ge;

  # named pages
  s!\[\[($pagePattern)\|([^\]\]]+?)\]\]!$this->pod_link($2 . "|" . $1)!ge;
   
  #  uri within brackets
  s!\[\[($linkProtocolPattern\:([^\s\<\>\"]+[^\s\.\,\!\?\;\:\)\<]))\]\]!$this->pod_link($1)!ge;

  # uri within brackets with description
  s!\[\[($linkProtocolPattern\:([^\s\<\>\"]+[^\s\.\,\!\?\;\:\)\<]))\|([^\]]+?)\]\]!$this->pod_link($4 . "|" .  $1)!ge;
 
  # images
  s/\{\{($pagePattern)\.(png|jpg|gif|jpeg|bmp|tiff|svg)\}\}/<img src="$config->{gfx}\/$1.$2" alt="$1"\/>/gs;

  # images with description
  s/\{\{($pagePattern)\.(png|jpg|gif|jpeg|bmp|tiff|svg)\|([^\}\}]+?)\}\}/<img src="$config->{gfx}\/$1.$2" title="$3" alt="$1"\/>/gs;
  
  # tables FIXME
 
  return $_;
}

sub gettable {
  my ($this, $head_cells, $row_rows) = @_;
  my $cols = scalar @{$head_cells};
  my $table = qq(<table border="0" cellspacing="3" cellpadding="3"><tr>\n);
  foreach my $head (@{$head_cells}) {
    my $align = $this->getalign($head);
    $table .= qq(<th align="$align" valign="top">$head</th>\n);
  }
  $table .= qq(</tr>\n);
  if(@{$row_rows}) {
    foreach my $row (@{$row_rows}) {
      my $cells = scalar @{$row};
      my $biggest;
      if($cells < $cols) {
        # need colspan, find biggest cell
        my @sizes = map { length($_) } @{$row};
        foreach my $size (@sizes) {
          if($size > $biggest) {
            $biggest = $size;
          }
        }   
      }
      $table .= qq(<tr>\n);
      foreach my $cell (@{$row}) {
        my $align = $this->getalign($cell);
        $table .= qq(<td align="$align" valign="top");
        if($biggest && length($cell) == $biggest) {
          my $span = ($cols - $cells ) + 1;
          $table .= qq( colspan="$span");
          $biggest = 0; # we support just ONE colspanned cell!
        }
        $table .= qq(><p>$cell</p></td>\n);
      }
      $table .= qq(</tr>\n);
    }
  }
  $table .= qq(</table>\n);
  return $table;
}

sub getalign {
  my($this, $cell) = @_;
  my $align = "left"; # default
  if($cell =~ /^(\s*).+?(\s*)$/) {
    my $left = length($1);
    my $right = length($2);
    if($left > $right) {
      $align = "right";
    }
    elsif( $left < $right) {
      $align = "left";
    }
    elsif( $left == $right ) {
      $align = "center";
    }
  }
  return $align;
}

sub html2pre {
  my ($this, $text) = @_;
  local $_ = $text;
  s/&/&amp;/g;
  s/</&lt;/g;
  s/>/&gt;/g;
  return $_;
}


1;
