 #
 #   $Id: Template.pm,v 1.4 2007/02/06 12:00:38 zarahg Exp $
 #
 #   This  file  is  part of the  PodWiki  web-based authoring tool.
 #
 #   By  accessing  this software,  PodWiki,  you are  duly informed
 #   of and  agree to be  bound  by the  conditions  described below
 #   in this notice:
 #
 #   This software product, PodWiki,  is developed by  Thomas Linden
 #   and   copyrighted  (C) 2003-2007   by  Thomas Linden,  with all
 #   rights reserved.
 #
 #   There is  no charge for  PodWiki software. You can redistribute
 #   it and/or modify it under  the terms of the  GNU General Public
 #   License, which is incorporated by reference herein.
 #
 #   PodWiki is distributed WITHOUT ANY WARRANTY,IMPLIED OR EXPRESS,
 #   OF  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that
 #   the  use of it will not infringe on any third party's intellec-
 #   tual property rights.
 #
 #   You  should  have  received a  copy  of  the GNU General Public
 #   License along with PodWiki.  Copies  can also be obtained from:
 #
 #     http://www.gnu.org/copyleft/gpl.html
 #
 #   or by writing to:
 #
 #     Free Software Foundation, Inc.
 #     59 Temple Place, Suite 330
 #     Boston, MA 02111-1307
 #     USA
 #
 #   Or contact:
 #
 #    "Thomas Linden" <tom@daemon.de>
 #
 #   Additional Copyrights:
 #
 #


#
# this module provides functions for template management
#


package PodWiki::Template;

$PodWiki::Template::VERSION = "1.00";



use lib qw(. lib);

use strict;
no strict "refs";

use Exporter;
use Data::Dumper;
use CGI::Cookie;
use PodWiki::Revision;

use vars qw(@ISA @EXPORT @EXPORT_OK @EXPORT_TAGS);

@ISA=qw(Exporter);
@EXPORT=qw(install_template install_templates remove_template);
@EXPORT_OK=qw();
@EXPORT_TAGS=();

sub install_templates {
  # always called from setup
  my $config = $PodWiki::Runner::config;
  my $input  = $PodWiki::Runner::input;

  opendir D, "$config->{templates}" or die "Could not read template directory: $!\n";
  foreach my $file ( readdir(D)) {
    if($file =~ /\.zip$/i) {
      &install_template("$config->{templates}/$file", 1);
    }
  }
  &install_template("$config->{templates}", 1);
}

sub install_template { 
  my($file, $silent) = @_;
                    
  my $config = $PodWiki::Runner::config;
  my $input  = $PodWiki::Runner::input;

  my ($sourcedir, $zipfile, $base, $dir);
  $base = "/tmp/$$";
  mkdir $base;

  if (exists $input->{file}) {
    # uploaded template tarball
    if ($input->{file} =~ /^(.*)\.zip$/i) {
      $dir = $1;
    }
    else {
      print qq(<p class="nolink">$input->{file} seems not to be a valid template file!</p>);
      return;
    }
    my $fh = $input->{file};
    open F, "> $base/$input->{file}" or die "Could not write file $base/$input->{file}: $!\n";
    binmode F;
    my $buffer;
    my $bytes;
    while ($bytes = read($fh,$buffer,1024)) {
      print F $buffer;
    }
    close F;
    $zipfile = "$input->{file}";
  }
  elsif ($file) {
    if(-d "$file") {
      $zipfile = "";
      $sourcedir = "$file";
    }
    else {
      if ($file =~ /^(.*)\.zip$/i) {
        $dir = $1;
      }
      else {
        print qq(<p class="nolink">$file seems not to be a valid template file!</p>);
        return;
      }
      system ("cp", "$file", "$base") and die "Could not copy $file to tmp directory: $!\n";
      $zipfile = "$file";
      $zipfile =~ s(.*/)();
    }
  }
  else {
    print qq(<p class="nolink">no ZIP file given!</p>);
    return;
  }

  if($zipfile) {
    system ("cd $base && unzip -q $base/$zipfile") and die "Could not unzip $base/$zipfile: $!\n";
    if(! -f "$base/info.txt") {
      print qq(<p class="nolink">$dir doesn't contain a info.txt file!</p>);
      return;
    }
    else {
      $sourcedir = $base;
    }
    system("rm $base/$zipfile");
  }

  # create the content
  my $autoloader = "";
  my $name       = "";
  my $version    = "";
  my @files      = ();
  my @datafiles  = ();
  my @images     = ();
  opendir D, "$sourcedir" or die "Could not read template directory: $!\n";
  foreach my $file ( readdir(D) ) {
    if ($file =~ /^.pod.*/) {
      if($file =~ /(podusers|podgroups)/) {
        # ignore security files
        next;
      }
      else {
        open POD, &PodWiki::Runner::untaint("$sourcedir/$file") or die "Could not open .pod* file: $!\n";
        my $string = join "", <POD>;
        close POD;
        PodWiki::Struct::restore($string, "$config->{data}/$file");
      }
    }
    elsif ($file =~ /^info.txt/) {
      # load template preferences
      open INFO, "<$sourcedir/info.txt" or die "Could not open info.txt file: $!\n";
      while (<INFO>) {
        chomp $_;
        if(/^autoloader = (.*)$/) {
          $autoloader = $1;
        }
        if(/^name = (.*)$/) {
          $name = $1;
        }
        if(/^files = (.*)$/) {
          @files = split /\s*,\s*/, $1;
        }
        if(/^images = (.*)$/) {
          @images = split /\s*,\s*/, $1;
        }
        if(/^version \s*=\s*(.*)$/) {
          $version = $1;
        }
      }
      close INFO;
    }
    elsif ($file =~ /^\./) { next; }
    else {
      push @datafiles, $file;
    }
    if(! $silent) {
      print qq(<b>$file</b><br/>);
    }
  }
   
  if((!$autoloader || !$name || !@files || !$version) && $zipfile) {
    print qq(<p class="nolink">Template $zipfile's info.txt is incomplete!</p>);
    return;
  }
  
  my $update = 0;
  if($zipfile) { 
    &PodWiki::Runner::tie_templates;
    if(exists $PodWiki::Runner::templates{$name}) {
      if($PodWiki::Runner::templates{$name}->{version} < $version) {
        $update = 1;
      }
      else {
        print qq(<p class="nolink">An older version of template "$name" is already installed: $PodWiki::Runner::templates{$name}->{version}!</p>);
        return;
      }
    }
  }
   
  if(! $zipfile) {
    # use all files which includes docs during 1st install
    @files = @datafiles;
  }
 
  my %index;  
  foreach my $file (@files) {
    next if($file =~ /\.zip$/);

    open F, "<$sourcedir/$file" or die "Could not read $sourcedir/$file: $!\n";
    my $content = join "", <F>;
    close F;


    my $rcs = new PodWiki::Revision(@PodWiki::Runner::rcs, "$file");

    if($update) {
      $rcs->LOCK($PodWiki::Runner::current_user);
    }

    system("cp", &PodWiki::Runner::untaint("$sourcedir/$file"), &PodWiki::Runner::untaint("$config->{data}/$file")) and
         die "Could not install template file $file: $!\n";

       if($update) {
         $rcs->commit("Updated to version $version", "$PodWiki::Runner::current_user", "Final");
       }
       else {
         $rcs->add('InitialCheckinByPodWikiSetup');
         $rcs->checkout(); 
       }
       $content =~ s/X<([^>]+?)>/&indexize($1, $file)/ge;
  }

  if(%PodWiki::Runner::index) {
     &PodWiki::Struct::store(\%PodWiki::Runner::index, "$config->{data}/.podindex");
  }

  if(@images) {
    foreach my $file (@images) {
      system("cp", &PodWiki::Runner::untaint("$sourcedir/$file"), &PodWiki::Runner::untaint("$config->{imgdir}/$file")) and
        die "Could not install template image $file: $!\n";
    }
  }

  if($zipfile) { 
    $PodWiki::Runner::templates{$name} = {
          autoloader => $autoloader,
          files => join(", ", @files),
          images => join(", ", @images),
          version => $version
    };
    &PodWiki::Runner::save_templates;
  }

  if($sourcedir =~ /^\/tmp/) {
    system("rm -rf /tmp/$$");
  }
  
  if(! $silent) {
    my $what = "installed";
    if($update) { $what = "updated to version $version"; }
    print qq(<p class="nolink">Template $name successfully installed.</p>);
  }
}

sub indexize {
  my($x, $file) = @_;
  foreach my $word (split /\s\s*/, $x) {
    if (exists $PodWiki::Runner::index{$word}) {
      # add current page to this word
      my $H = $PodWiki::Runner::index{$word};
      $H->{$file} = 1;
      $PodWiki::Runner::index{$word} = $H;
    }
    else {
      # add the word (along with current page)
      $PodWiki::Runner::index{$word} = { $file => 1 };
    }
  }
  return "";
}

sub remove_template {
  &PodWiki::Runner::tie_templates;
  my %templates = %PodWiki::Runner::templates;
  my $input     = $PodWiki::Runner::input;
  my $config    = $PodWiki::Runner::config;
  if(exists $templates{$input->{template}}) {
     foreach my $file (split /\s*,\s*/, $templates{$input->{template}}->{files}) {
       unlink "$config->{data}/$file";
       unlink "$config->{data}/$file,v";
       unlink "$config->{data}/$file,lockuser";
     }
     foreach my $file (split /\s*,\s*/, $templates{$input->{template}}->{images}) {
       unlink "$config->{imgdir}/$file";
     }
     delete $PodWiki::Runner::templates{$input->{template}};
     &PodWiki::Runner::save_templates;
     print qq(<p class="nolink">Template $input->{template} has been removed.</p>);
  }
  else {
    print qq(<p class="nolink">Template $input->{template} is not installed!</p>);
  }
  &PodWiki::Runner::site_admin;
  return;
}

1;
